#!/usr/bin/python
#
# $Id: imapcplib.py,v 1.3 2002/06/08 00:03:02 louisi Exp $
#
# imap tool library
#
# This software is released under the GNU GPL. See the LICENSE file for
# details.  This  software   copyright  (c)   2001  by   Jean-Philippe   
# Langlois (jpl@iname.com).

import imaplib
import string
import sys
import re
import base64
import getpass

enable_debug = 0

enable_lsub  = 0

def usage():
        print "usage: %s [-v] [-s] < file" % ( sys.argv[0] )
        print "where:"
        print "\t-s:      copy subscribed mailboxes only"
        print "\t-v:      enable debug messages"
        print

def debug( str ):
	if enable_debug:
		sys.stderr.write( str + "\n" )
		sys.stderr.flush()

def parse_mailbox( str ):
        # format: (\flag \flag ...) "sep" mailbox-name
        r = re.compile( '^\\((.*)\\) "(.)" (.*)$' )
        match = r.match( str )
        if match is None:
                debug( "Mailbox string did not match format:" + str )
                return

        flags = string.split( match.group( 1 ), ' ' )

        return ( flags, match.group( 2 ), match.group( 3 ) )

def show_mailboxes( mailboxes ):
	for mailbox in mailboxes:
		flags, sep, name = parse_mailbox( mailbox )
		debug( "%s (%s, %s)" % ( name, sep, flags ) )

	# return seperator for info
	return sep

def parse_fetch( data ):
	# data[0][0] contains something like:
	# 1 (FLAGS (\\Seen) INTERNALDATE " 4-Oct-1997 11:12:25 -0500" RFC822 {664}
	# message is actually in data[0][1]

	r = re.compile( '^(.*) \\(FLAGS \\((.*)\\) INTERNALDATE "(.*)" .*$' )
	match = r.match( data[0][0] )
	if match is None:
		debug( "Message fetch format error " + data[0][0] )
		return

	num = match.group( 1 )
	flags = match.group( 2 )
	internaldate = match.group( 3 )
	message = data[0][1]

	debug( "message fetched: %s, %s, %s" % ( num, flags, internaldate ) )

	return (num, flags, internaldate, message )

# this mailbox name stuff does not work... it seems to be from the IMAP RFC... but the server
# complains.
def string_encode( str ):
	return "&" + string.replace( base64.encodestring( str ), "/", "," ) + "-"

def convert_mailbox_name( namea, roota, sepa, rootb, sepb ):

	# For VMSG we have a special case
	if namea == 'INBOX':
		return rootb

	# split into individual folder names
	name = string.split( namea, sepa )

	if name[0] == roota:
		name = name[1:]

	# 3 verify that new separator is not in mail box names
	name2 = [rootb]

	if sepb != "_": rep = "_"
	else: rep = "."

	for f in name:
		f2 = string.replace( f, sepb, rep )
		name2.append( f2 )

	nameb = string.join( name2, sepb )

	return nameb


