<?php
/*
 * Copyright (C) 2001-2002 Caldera International, Inc.  All Rights Reserved.
 */

/******************************************************************************
 * Class Alias
 *
 * Encapsulates all the attributes and methods pertaining to an email alias.
 *
 * Public Methods:
 *	Alias() polymorphic constructor
 *
 ******************************************************************************
 */
class Alias {
	/* Properties */

	// sv = single valued, mv = multivalued
	var $mail;	// sv, Email address (with domain), *required*
	var $name;      // sv, Email without the domain, forced value
	var $domain;	// mv, domain which the alias belongs to
	var $description; // sv, description of the alias purpose
	var $joinable;	// sv, boolean, non-owner can add and delete
	var $owners;	// mv, array of user mail addresses who are owners
	var $ouids;	// mv, array of user mail addresses who are owners
	var $programs;	// mv, array of programs called on deliver
	var $files;	// mv, array of files appended on deliver
	var $mgenerics;	// mv, array of mail addresses (receipients)
	var $musers;	// mv, array of mail addresses (receipients)
	var $muids;	// mv, array of mail addresses (receipients)
	var $maliases;	// mv, array of mail addresses alias receipients
	var $mremotes;	// mv, array of mail addresses for non-domain members

	/* Methods */

	/**********************************************************************
	 * Public Method Alias()
	 *
	 * This is the constructor for the Alias class. It can be used to
	 * create an empty alias obj, an alias obj with all the properties
	 * of a an existing alias, or an alias obj with the properties 
	 * specified in a argument array (usually provided from screen form).
	 * 
	 * Parameters & Return:
	 * The constructor can be called with the following parameters:
	 * o none - returns empty alias object
	 * o arg array - key value pairs of alias properties
	 * o
	 **********************************************************************
	 */
	function Alias() {
		$numArgs = func_num_args();

		switch ($numArgs) {
		  case 0:
			$this->_emptyAlias();
			break;
		  case 1:
			$arg1 = func_get_arg(0);
			if (is_array($arg1)) {
				$ret = $this->_createAlias($arg1);
				if (! $ret) {
					$this = '';
				}
			} else {
				$this->_fetchAlias($arg1); // mail or name
			}
			break;
		  case 2:
			$arg1 = func_get_arg(0); // mail or name
			$arg2 = func_get_arg(2); // array of attributes
			$this->_fetchAlias($arg1, $arg2);
			break;
		  default:
			$this = "";
			break;
		}

	}
	/* Methods */

	function _fetchAlias($mail, $argArray = '') {
		global $session_domain;

		if ($mail == '') {
			$this = '';
			return;
		}

		$pos = strpos($mail, '@');
		if ($pos === false) {
			$mail .= '@' . $session_domain;
		}

		$attrs = ' --all';

		$cmd = new Command('msgaliaslist');
		$cmd->add_arg('--mail', $mail);
		$cmd->add_arg($attrs);
		$cmd->run();
		$lines = $cmd->getOutput();

		if ($lines == '') {
			$this = '';
			return;
		}

		$count = 0;
		while ($line = array_shift($lines)) {
			$line = trim($line);

			if ($line != '') {
				list($attr, $value) = split(':', $line, 2);
				switch ($attr) {
					case 'mail' :
						$count++;
						$this->mail = $value;
						break;
					case 'name' :
						$this->name = $value;
						break;
					case 'domain' :
						$count++;
						$this->domain = $value;
						break;
					case 'description' :
						$count++;
						$this->description = $value;
						break;
					case 'member' :
						$count++;
						$savedValue = $value;
						break;
					case 'memberalias' :
						$count++;
						$this->maliases[] = $savedValue;
						unset($savedValue);
						break;
					case 'memberuid' :
						$count++;
						$this->muids[] = $value;
						$this->musers[] = $savedValue;
						unset($savedValue);
						break;
					case 'remote' :
						$count++;
						$this->mremotes[] = $value;
						break;
					case 'owner' :
						$count++;
						$savedValue = $value;
						break;
					case 'owneruid' :
						$count++;
						$this->ouids[] = $value;
						$this->owners[] = $savedValue;
						unset($savedValue);
						break;
					case 'file' :
						$count++;
						$this->files[] = $value;
						break;
					case 'program' :
						$count++;
						$this->programs[] = $value;
						break;
					case 'joinable' :
						$count++;
						if (strcasecmp($value, 'true') == 0) {
							$this->joinable = true;
						} else {
							$this->joinable = false;
						}
						break;
					default:
						break;
				}
			}
		}

		if (! $count > 0 ) {
			$this = '';
			return;
		}
	}

	function _emptyAlias() {
		; // nothing
	}

	function _createAlias($varArray) {
		global $session_domain;

		$cmd = new Command('msgaliascreate');
		$cmd->add_arg('--domain', $session_domain);
		$varArray['domain'] = $session_domain;
		unset($GLOBALS['LAST_ERROR']);

		while (list ($attr, $val) = each ($varArray)) {
			$skip = false;

			$val = trim($val);

			if ($val == '')  continue;

			$attr = str_replace('_', '', $attr);

			if (! $this->_validate($attr, $val)) continue;

			switch ($attr) {
				case 'mail' :
					//ignore
					continue;
					break;
				case 'name' :
				    $this->name = $val;
				    break;
				case 'owner' :
				    $a = $this->_parseAddressList($val,
					'owner', 'create');
				    $e_owner = $this->_getAddressErrors($a,
					'owner', 'create');

				    $opt = $this->_getOption($attr);
				    if ($a['good']) foreach ($a['good'] as $m) {
					$cmd->add_arg($opt, $m);
					$this->owners[] = $m;
				    }

				    $skip = true;
				    break;
				case 'mgenerics' :
				    $a = $this->_parseAddressList($val, 
					'member', 'create');
				    $e_member = $this->_getAddressErrors($a,
					'member', 'create');

				    $opt = $this->_getOption($attr);
				    if ($a['good']) foreach ($a['good'] as $m) {
					$cmd->add_arg($opt, $m);
					$this->mgenerics[] = $m;
				    }

				    $skip = true;
				    break;
				case 'file':
				    $this->files = $val;
				    break;
				case 'program':
				    $this->programs = $val;
				    break;
				case 'joinable':
				    $val = strtoupper($val);
				    break;
				default:
				    break;
			}

			if (! $skip) {
				$opt = $this->_getOption($attr);
				if ($opt != '') {
					$cmd->add_arg($opt, $val);
				}
			}
		}

		if (! isset($this->name)) {
			$GLOBALS['LAST_ERROR'][] = 
				getIntlMsg('ERR_ALIAS_NO_ALIAS_SPECIFIED');
			$GLOBALS['LAST_ERROR'][] = 
				getIntlMsg('ERR_ALIAS_EMAIL_REQUIRED');
			return false;
		}
		if (! isset($this->owners)) {
			$GLOBALS['LAST_ERROR'][] = 
				getIntlMsg('ERR_ALIAS_NO_OWNER');
			$GLOBALS['LAST_ERROR'][] = 
				getIntlMsg('ERR_ALIAS_OWNER_REQUIRED');
			if ($e_owner)
			    $GLOBALS['LAST_ERROR'] = 
				array_merge($GLOBALS['LAST_ERROR'], $e_owner);
			
			return false;
		}
		if (! ($this->mgenerics || $this->files || $this->programs)) {
			$GLOBALS['LAST_ERROR'][] = 
				getIntlMsg('ERR_ALIAS_NO_RECEIPIENT');
			$GLOBALS['LAST_ERROR'][] = 
				getIntlMsg('ERR_ALIAS_RECEIPIENT');
			if ($e_member)
			    $GLOBALS['LAST_ERROR'] = 
				array_merge($GLOBALS['LAST_ERROR'], $e_member);
			return false;
		}

		$cmd->run();

		if ($cmd->getExitVal() != 0) {
			$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
			$GLOBALS['LAST_ERROR'][] = 
				getIntlMsg('ERR_CMD_ON', $cmd->concat());
			$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
			return false;
		} elseif ($e_owner || $e_member) {
			$GLOBALS['LAST_ERROR'][] = 
				getIntlMsg('ERR_ALIASES_ALIAS_CREATED_BUT');

			if ($e_owner)
			    $GLOBALS['LAST_ERROR'] = 
				array_merge($GLOBALS['LAST_ERROR'], $e_owner);

			if ($e_member)
			    $GLOBALS['LAST_ERROR'] = 
				array_merge($GLOBALS['LAST_ERROR'], $e_member);

			$retval = false;
		} else {
			$retval = true;
		}

		unset($this->mgenerics);
		unset($this->owners);
		unset($this->programs);
		unset($this->files);
		unset($this->maliases);
		unset($this->musers);
		unset($this->mremotes);
		$this->_fetchAlias($this->name);
	
		return $retval;
	}

	function modify($varArray) {

		$cmd = new Command('msgaliasreplace');
		$cmd->add_arg('--mail', $this->mail);
		$current = get_object_vars($this);

		while (list ($attr, $val) = each ($varArray)) {

			$skip = false;

			$val = trim($val);

			$attr = str_replace('_', '', $attr);

			if ($current[$attr] == $val && $attr != 'joinable') {
				continue;
			}

			if (! $this->_validate($attr, $val)) continue;

			switch ($attr) {
				case 'mail' :
				case 'name' :
				case 'domain' :
				case 'owners' :
				case 'programs' :
				case 'files' :
				case 'mgenerics' :
				case 'maliases' :
				case 'muids' :
				case 'mremotes' :
				    continue; // none of these can be modified
					       // with this function
				case 'joinable':
				    $val = strtoupper($val);
				    break;
				default: 
				    break;
			}

			if (! $skip) {
				$opt = $this->_getOption($attr);
				if ($opt != '') {
					$cmd->add_arg($opt, $val);
				}
			}
		}

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			$this = new Alias($this->mail);
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function delete() {
	
		$cmd = new Command('msgaliasdelete');
		$cmd->add_arg('--mail', $this->mail);
		$cmd->add_arg('--recurse');
		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function isRequired($attr) {
		switch ($attr) {
			case 'mail':
			case 'name':
			case 'owner':
			case 'member':
				$retval = true;
				break;
			case 'domain':
			case 'joinable':
			case 'description':
			case 'file':
			case 'program':
			default:
				$retval = false;
				break;
		}
		return $retval;
	}

	function _validate($attr, $value) {
		return true;
	}

	function _getOption($attr) {

		$attr = str_replace('_', '', $attr);

		switch ($attr) {
			case 'mail' :
				$retval = '';
				break;
			case 'name' :
				$retval = '--name';
				break;
			case 'description':
				$retval = '--description';
				break;
			case 'member' :
			case 'mgenerics' :
				$retval = '--member';
				break;
			case 'owner' :
				$retval = '--owner';
				break;
			case 'joinable' :
				$retval = '--joinable';
				break;
			case 'remote' :
				$retval = '--remote';
				break;
			case 'file' :
				$retval = '--file';
				break;
			case 'program' :
				$retval = '--program';
				break;
			default:
				$retval = '';
		}

		return $retval;
	}

	function setAttr($attr, $val) {
		switch ($attr) {
			case 'mail':
				$this->mail = $val;
				$this->name = 
					substr($val, 0, strpos($val, '@'));
				break;
			case 'description':
				$this->description = $val;
				break;
			default:
				break;
		}
	}

	function setMail($value) {
		$this->mail = $value;
		$this->name = substr($value, 0, strpos($value, '@'));
	}

	function getMail() {
		return $this->mail;
	}

	function getName() {
		return $this->name;
	}

	function getDescription() {
		return $this->description;
	}

	function isJoinable() {
		return $this->joinable;
	}

	function isOwnerUID($uid) {
		$tmp = new User($uid);
		if (count(array_intersect($this->ouids, array($uid))) == 1) {
			return true;
		}
		return false;
	}

	function isMemberUID($uid) {
		if (count(array_intersect($this->muids, array($uid))) == 1) {
			return true;
		}
		return false;
		
	}

	function isMember($a) {
		$found = false;

		if (! is_object($a)) {
			$a = new MailAddress($a);
		}

		if (preg_match('/mailaddress/i', get_class($a))) {

		    switch ($a->type) {
			case 'remote':
			    if ($this->mremotes && 
				in_array($a->mail, $this->mremotes)) {
				$found = true;
			    }
			    break;
			case 'user':
			    if ($this->musers && 
				in_array($a->mail, $this->musers)) {
				$found = true;
			    }
			    break;
			case 'alias':
			    if ($this->maliases && 
				in_array($a->mail, $this->maliases)) {
				$found = true;
			    }
			    break;
			default:
			    break;
		    }
		}

		return $found;
	}

	function isOwner($a) {
		$found = false;

		if (! is_object($a)) {
			$a = new MailAddress($a);
		}

		if (preg_match('/mailaddress/i', get_class($a))) {

		    switch ($a->type) {
			case 'user':
			    if ($this->owners &&
				in_array($a->mail, $this->owners)) {
				$found = true;
			    }
			    break;
			default:
			    break;
		    }
		}

		return $found;
	}

	function comparisonFunctionName($field, $direction) {
		switch($field) {
			case 'name':
				$cmpFunction = 'byAliasName';
				break;
			case 'mail':
				$cmpFunction = 'byAliasMail';
				break;
			case 'description':
				$cmpFunction = 'byAliasDescription';
				break;
			default:
				$cmpFunction = 'byAliasName';
				break;
		}

		switch($direction) {
			case 'a':
				$cmpFunction .= 'Ascend';
				break;
			case 'd':
				$cmpFunction .= 'Decend';
				break;
			default:
				$cmpFunction .= 'Acend';
				break;
		}
		return $cmpFunction;
	}

	function match($expr, $attr = '') {
		$expr = substr($expr, 0, 256);
		if ($attr != '') {
			$retval = @ereg($expr, $this->$attr);
		} else {
			if (@eregi($expr, $this->name)) {
				$retval = true;
			} else {
				$retval = false;
			}
		}
		return $retval;
	}

	function _splitMembers($slist) {
		if (ereg('"', $slist)) {
			$GLOBALS['LAST_ERROR'][] = 
				getIntlMsg('ERR_ALIAS_NO_QUOTES',
				getIntlMsg('ERR_ALIAS_QUOTED_ADDRESS'));
			$GLOBALS['LAST_ERROR_EXIT'] = '';
			return false;
		}

		$slist   = strtr($slist, ",\n\r\t\0", ' ');
		$slist   = ereg_replace("[\n\r]", ' ', $slist);
		$slist   = trim($slist);

		$alist = split("[ ]+", $slist);
		$alist = array_unique($alist);
		
		return $alist;
	}

	/**********************************************************************
	 * function _parseAddressList
	 *
	 * parameters:
	 * o $list - string - list of mail addresses to process into cmd
	 * o $attr - string - member, mgenerics or owner
	 * o $op - string - add, remove, create
	 * returns:
	 * o array of array or strings - array elements are:
	 * 	- good, badaddress, badowner, badremote, badalias
	 *	each element is an array or addresses that fall in that
	 *	category, if an element doesn't exisit, no addresses matched
	 *	that category.
	 * o will return false if no addresses could be parsed
	 **********************************************************************
	 */
	function _parseAddressList($list, $attr, $op) {
		global $session_perms;

		$addresses = $this->_splitMembers($list);
		if ($addresses === false) {
			return false;
		}

		// sort addresses into good, bad and ugly
		foreach ($addresses as $address) {
			$a = new MailAddress($address);
			$found = false;

			if (! $a) {

				$results['badaddress'][]  = $address;
				continue;

			}

			// check requirements for owner
			if ($attr == 'owner' && $a->type != 'user') {

				$results['badowner'][] = $a->mail;
				continue;

			}

			// check can add remote member acl
			if ($a->type == 'remote' &&
			    ! isAdmin() &&
			    ! $session_perms->canAddRemoteMember()) {

				$results['badremote'][] = $a->mail;
				continue;

			}

			// check can add alias member acl
			if ($a->type == 'alias' &&
			    ! isAdmin() &&
			    ! $session_perms->canAddAliasMember()) {

				$results['badalias'][] = $a->mail;
				continue;

			} 

			if ($attr == 'owner') {
				if ($this->isOwner($a->mail)) {
					$results['owner'][] = $a->mail;
					$found = true;
				} else {
					$results['nonowner'][] = $a->mail;
				}
			} elseif ($attr == 'member' || $attr == 'mgenerics') {
				if ($this->isMember($a->mail)) {
					$results['member'][] = $a->mail;
					$found = true;
				} else {
					$results['nonmember'][] = $a->mail;
				}
			}

			if ($op == 'remove' && $found) {
				$results['good'][] = $a->mail;
			} elseif ($op != 'remove' && ! $found)  {
				$results['good'][] = $a->mail;
			}

		}

		return $results;
	}

	function _getAddressErrors($a, $target, $op) {

		if ($a['badaddress']) {
			$results[] =
				getIntlMsg('ERR_ALIASES_BAD_ADDRESSES',
					join(' ', $a['badaddress']));
		}

		if ($target == 'owner' && $a['badowner']) {
			$results[] = 
				getIntlMsg('ERR_ALIASES_BAD_OWNERS',
			  		join(' ', $a['badowner']));
		}

		if($a['badremote'] && $a['badalias']) {
			$b = array_merge($a['badremote'], $a['badalias']);
		} elseif ($a['badremote']) {
			$b = $a['badremote'];
		} elseif ($a['badalias']) {
			$b = $a['badalias'];
		} else {
			$b = false;
		}
		if ($b) {
			$results[] = 
				getIntlMsg('ERR_ALIASES_CANT_WRITE_PRIV_MEMBER',
					join(' ', $b));
		}

		if ($op == 'remove') {
		    if ($target == 'member' && $a['nonmember']) {
			$results[] = 
				getIntlMsg('ERR_ALIASES_NOT_A_MEMBER',
				join(' ', $a['nonmember']));
		    } elseif ($target == 'owner' && $a['nonowner']) {
			$results[] = 
				getIntlMsg('ERR_ALIASES_NOT_AN_OWNER',
				join(' ', $a['nonmember']));
		    }
		} else {
		    if ($target == 'member' && $a['member']) {
			$results[] = 
				getIntlMsg('ERR_ALIASES_ALREADY_A_MEMBER',
				join(' ', $a['member']));
		    } elseif ($target == 'owner' && $a['owner']) {
			$results[] = 
				getIntlMsg('ERR_ALIASES_ALREADY_AN_OWNER',
				join(' ', $a['owner']));
		    }
		}

		if ($results) {

			if ($a['good']) {
				$results[] = getIntlMsg(
					($op == 'remove') ? 
					'ERR_ALIASES_VALID_ADDRESSES_REMOVED' :
					'ERR_ALIASES_VALID_ADDRESSES_ADDED',
					join(' ', $a['good']));
			}

			array_unshift($results,
				getIntlMsg(($target == 'owner') ?
				'ERR_ALIASES_OWNER_ADDRESSES' :
				'ERR_ALIASES_MEMBER_ADDRESSES'));

			return $results;
	
		} else {

			return false;

		}
	}

	function _addressAction($list, $cmd, $attr, $op) {

		$a = $this->_parseAddressList($list, $attr, $op);
		$a_errors = $this->_getAddressErrors($a, $attr, $op);

		if (! $a) return false;
		$opt = $this->_getOption($attr);
		if ($a['good']) foreach ($a['good'] as $m) {
			$cmd->add_arg($opt, $m);
		} else {
			if ($a_errors) $GLOBALS['LAST_ERROR'] = $a_errors;
			$GLOBALS['LAST_ERROR_EXIT'] = '';
			return false;
		}

		$cmd->run();

		if ($cmd->getExitVal() != 0) {
			$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
			$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON',
				$cmd->concat());
			$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
			return false;
		} elseif ($a_errors) {
			$GLOBALS['LAST_ERROR'] = $a_errors;
			$GLOBALS['LAST_ERROR_EXIT'] = '';
			return false;
		} else {
			return true;
		}

	}

	function addMember($member) {
		global $session_user;

		// can add self if alias is joinable
		if ($member == $session_user && $this->joinable) {
			return $this->addMembers($member);
		}

		// if not joinable or not the member, can add if admin
		if (isAdmin()) {
			return $this->addMembers($member);
		}

		// if not any of those, can add if the owner
		if ($this->isOwner($session_user)) {
			return $this->addMembers($member);
		}

		// can't do it
		unset($GLOBALS['LAST_ERROR'][]);
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_NO_PERMISSION');
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_OWNER_OR_ADMIN');
		return false;
	}

	function removeMember($member) {
		global $session_user;

		// can add self if alias is joinable
		if ($member == $session_user && $this->joinable) {
			return $this->removeMembers($member);
		}

		// if not joinable or not the member, can add if admin
		if (isAdmin()) {
			return $this->removeMembers($member);
		}

		// if not any of those, can add if the owner
		if ($this->isOwner($session_user)) {
			return $this->removeMembers($member);
		}

		// can't do it
		unset($GLOBALS['LAST_ERROR'][]);
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_NO_PERMISSION');
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_OWNER_OR_ADMIN');
		return false;
	}

	function addMembers($list) {

		$cmd = new Command('msgaliasadd');
		$cmd->add_arg('--loop');
		$cmd->add_arg('--mail', $this->mail);
		return $this->_addressAction($list, $cmd, 'member', 'add');

	}

	function removeMembers($list) {

		$cmd = new Command('msgaliasremove');
		$cmd->add_arg('--mail', $this->mail);
		return $this->_addressAction($list, $cmd, 'member', 'remove');

	}

	function addOwners($list) {

		$cmd = new Command('msgaliasadd');
		$cmd->add_arg('--mail', $this->mail);
		return $this->_addressAction($list, $cmd, 'owner', 'add');

	}

	function removeOwners($list) {

		$cmd = new Command('msgaliasremove');
		$cmd->add_arg('--mail', $this->mail);
		return $this->_addressAction($list, $cmd, 'owner', 'remove');

	}

	function _isValidFileProg($s) {
		if (strpos($s, '/') === 0) {
			return true;
		}
		return false;
	}

	function addFile($s) {
		if (! isAdmin()) {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_AUTH_NO_MODIFIY_PERM'));
			return false;
		}

		if (! $this->_isValidFileProg($s)) {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_ALIAS_INVALID_FILE', $s));
			return false;
		}
		$cmd = new Command('msgaliasadd');
		$cmd->add_arg('--mail', $this->mail);
		$cmd->add_arg('--file', $s);

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function addProgram($s) {
		if (! isAdmin()) {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_AUTH_NO_MODIFIY_PERM'));
			return false;
		}

		if (! $this->_isValidFileProg($s)) {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_ALIAS_INVALID_PROGRAM', $s));
			return false;
		}
		$cmd = new Command('msgaliasadd');
		$cmd->add_arg('--mail', $this->mail);
		$cmd->add_arg('--program', $s);

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function removeFile($s) {
		if (! isAdmin()) {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_AUTH_NO_MODIFIY_PERM'));
			return false;
		}

		if (! in_array($s, $this->files)) {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_ALIAS_NO_SUCH_FILE', $s));
			return false;
		}

		$cmd = new Command('msgaliasremove');
		$cmd->add_arg('--mail', $this->mail);
		$cmd->add_arg('--file', $s);

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function removeProgram($s) {
		if (! isAdmin()) {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_AUTH_NO_MODIFIY_PERM'));
			return false;
		}

		if (! in_array($s, $this->programs)) {
			$GLOBALS['LAST_ERROR'] = array(
				getIntlMsg('ERR_ALIAS_NO_SUCH_PROGRAM', $s));
			return false;
		}

		$cmd = new Command('msgaliasremove');
		$cmd->add_arg('--mail', $this->mail);
		$cmd->add_arg('--program', $s);

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

}

/*
 * Ideally these comparison functions would be encapsulated in the 
 * Alias class as class methods. Can't work out how to pass a class
 * method to the usort() function yet.
 */
function byAliasNameAscend($left, $right) {
	return(strcasecmp($left->name,$right->name));
}

function byAliasNameDecend($left, $right) {
	return(-1 * (byAliasNameAscend($left, $right)));
}

function byAliasMailAscend($left, $right) {
	return(strcasecmp($left->mail,$right->mail));
}

function byAliasMailDecend($left, $right) {
	return(-1 * (byAliasMailAscend($left, $right)));
}

function byAliasDescriptionAscend($left, $right) {
	return(strcasecmp($left->description,$right->description));
}

function byAliasDescriptionDecend($left, $right) {
	return(-1 * (byAliasDescriptionAscend($left, $right)));
}

?>
