<?php
/*
 * Copyright (C) 2001-2002 Caldera International, Inc.  All Rights Reserved.
 */

class AliasSelectList extends ObjectTable {

	function AliasSelectList($varArray, $id) {
		$domain = $GLOBALS['session_domain'];
		$this->list = '';
		$cmd = new Command('msgaliaslistall');
		$cmd->add_arg('--domain', $domain);
		$cmd->add_arg('--mail --description');
		$cmd->run();
		$lines = $cmd->getOutput();

		unset($this->list);

		// Populate the list 
		if ($lines != '') foreach ($lines as $line) {

			$line = trim($line);

			if ($line == '') {
				if ($alias != '') {
					$this->list[] = $alias;
					$alias = '';
				}
				continue;
			}

			if ($alias == '' && $line != '') {
				$alias = new Alias();
			}

			list($attr, $value) = split(':', $line, 2);
			$alias->setAttr($attr, $value);
		}

		// Apply filter if any
		$varArray['listClass'] = 'AliasSelectList';
		$this->setFilter($varArray, $id);
		if (isset($this->filter)) $this->applyFilter($this->filter);

		// Setup the table
		if (Table::isActive($varArray, $id)) {
			$table = Table::getActive();
			$table->setParams($varArray, $this->count());
		} else {
			$table = new Table($id);

			$name = getIntlMsg('MSG_ALIASES_HEADING_SMAIL');
			$table->addDataColumn($name, Column::ASCEND(), 'name');

			$name = getIntlMsg('MSG_ALIASES_HEADING_DESCRIPTION');
			$table->addDataColumn($name, Column::ASCEND(), 'description');

			$name = getIntlMsg('MSG_ALIASES_HEADING_ADD');
			$table->addDisplayColumn($name, 
				"return AliasAdd('%%ROWID%%')");

			$table->setParams($HTTP_GET_VARS, $this->count());
			$table->rows = 10;
			$table->setSortedColumn($table->firstSortedColumn());
		}

		// Apply sorting
		$attr = $table->getSortAttrName($table->getSortedColumn());
		$dir  = $table->getColumnSortDir($table->getSortedColumn());
		$this->sort(Alias::comparisonFunctionName($attr ,$dir));

		// Save table
		$this->table = $table;
		$this->table->save();
	}

	function getListValue($index, $col = '') {
		global $session_lang;

		// if now $col then return the row value (unique id)
		if ($col === '') {
			return $this->list[$index]->getMail();
		}

		switch ($col) {
			case '0':
			case 'name':
				$retval = $this->list[$index]->getName();
				break;
			case '1' :
			case 'description':
				$retval = $this->list[$index]->getDescription();
				// truncate the description
				$retval = substr($retval, 0, $GLOBALS['TABLE_DESCRIPTION_WIDTH']);
				break;
			case '2' :
				$retval = $this->table->getTemplate('addIcon');
				$retval = ereg_replace('%%LANG%%', $session_lang, $retval);
				$retval = stripslashes($retval);
				break;
			default:
				$retval = '';
				break;
		}

		return $retval;
	}

	/**********************************************************************
	 * class function getSearchAttrs()
	 *
	 * return an array containing the attributes that can be used
	 * to filter the results displayed in the table. The array has the
	 * internally meaningful attribute as the key and the displayable
	 * attribute name as the value;
	 **********************************************************************
	 */
	function getSearchAttrs() {
		return array(
			'name' => getIntlMsg('MSG_ALIASES_HEADING_NAME'),
			'description' => getIntlMsg('MSG_ALIASES_HEADING_DESCRIPTION'),
		);
	}

}

?>
