<?php
/*
 * Copyright (C) 2001-2002 Caldera International, Inc.  All Rights Reserved.
 */

class Server {

	// user attribute access
	var $passwdWrite;    // s, boolean
	var $profileWrite;   // s, boolean
	var $forwardWrite;   // s, boolean
	var $vacationWrite;  // s, boolean
	var $junkmailWrite;  // s, boolean

	// user alias actions
	var $aliasCreate;     // s, boolean
	var $aliasAddAlias;   // s, boolean
	var $aliasAddRemote;  // s, boolean

	// junk mail settings
	var $doRBLLookups;    // s, boolean

	// system default values
	var $spamFilterDefault;   // s, boolean
	var $spamDestroyDefault;  // s, boolean
	
	function Server() {
		$numArgs = func_num_args();
		if ($numArgs != 0) {
			return; // create an empty server object
		}

		$cmd = new Command('msgaclget');
		$cmd->run();
		$lines = $cmd->getOutput();

		if ($lines != '') foreach ($lines as $line) {
			$line = trim($line);

			if ($line != '') {
				list($attr, $value) = split(':', $line, 2);

				switch ($attr) {
					case 'AllowUserPasswordWrite' :
						$this->passwdWrite =
							toBool($value);
						break;
					case 'AllowUserAliasCreate' :
						$this->aliasCreate =
							toBool($value);
						break;
					case 'AllowUserConfigWrite' :
						$this->profileWrite =
							toBool($value);
						break;
					case 'DefaultSpamFilterEnabled' :
						$this->spamFilterDefault =
							toBool($value);
						break;
					case 'DefaultSpamDestroyEnabled' :
						$this->spamDestroyDefault =
							toBool($value);
						break;
					case 'EnableRblLookups' :
						$this->doRBLLookups =
							toBool($value);
						break;
					case 'AllowUserEmbedAlias' :
						$this->aliasAddAlias =
							toBool($value);
						break;
					case 'AllowUserRemoteWrite' :
						$this->aliasAddRemote =
							toBool($value);
						break;
					case 'AllowUserForwardWrite' :
						$this->forwardWrite =
							toBool($value);
						break;
					case 'AllowUserVacationWrite' :
						$this->vacationWrite =
							toBool($value);
						break;
					case 'AllowUserSpamWrite' :
						$this->junkmailWrite =
							toBool($value);
						break;
					default:
						break;
				}
			}
		}
	}

	function _validate($attr, $val) {
		$retval = false;

		if (	preg_match('/true/i', $val)  || 
			preg_match('/false/i', $val)	) {

			$retval = true;
		}

		return $retval;
	}

	function _getOption($attr) {
		$attr = str_replace('_', '', $attr);

		switch ($attr) {
			case 'passwdWrite' :
				$retval = '--AllowUserPasswordWrite';
				break;
			case 'aliasCreate' :
				$retval = '--AllowUserAliasCreate';
				break;
			case 'profileWrite' :
				$retval = '--AllowUserConfigWrite';
				break;
			case 'forwardWrite' :
				$retval = '--AllowUserForwardWrite';
				break;
			case 'vacationWrite' :
				$retval = '--AllowUserVacationWrite';
				break;
			case 'junkmailWrite' :
				$retval = '--AllowUserSpamWrite';
				break;
			case 'aliasAddAlias' :
				$retval = '--AllowUserEmbedAlias';
				break;
			case 'aliasAddRemote' :
				$retval = '--AllowUserRemoteWrite';
				break;
			case 'doRBLLookups' :
				$retval = '--EnableRblLookups';
				break;
			case 'spamFilterDefault' :
				$retval = '--DefaultSpamFilterEnabled';
				break;
			case 'spamDestroyDefault' :
				$retval = '--DefaultSpamDestroyEnabled';
				break;
			default:
				$retval = '';
		}

		return $retval;
	}

	function modify($varArray) {
		$cmd = new Command('msgaclset');

		if ($GLOBALS['LDAPHost'] != 'localhost') {
			$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_ACLSET_NOTLOCALHOST');
			return false;
		}

		$current = get_object_vars($this);

		while (list ($attr, $val) = each ($varArray)) {

			$val = trim($val);

			$attr = str_replace('_', '', $attr);

			if (! $this->_validate($attr, $val)) continue;

			if ($current[$attr] === $val) continue;

			$opt = $this->_getOption($attr);
			$val = strtoupper($val);
			if ($opt !== '') {
				$cmd->add_arg($opt, $val);
			}
		}

		$cmd->run();

		if ($cmd->getExitVal() == 0) {
			$this = new Server();
			return true;
		}
		$GLOBALS['LAST_ERROR'] = $cmd->getErrors();
		$GLOBALS['LAST_ERROR'][] = getIntlMsg('ERR_CMD_ON', $cmd->concat());
		$GLOBALS['LAST_ERROR_EXIT'] = $cmd->getExitVal();
		return false;
	}

	function formExport() {
		$attrs = get_object_vars($this);
		foreach ($attrs as $attr => $val) {
			$GLOBALS["_$attr"] = $val;
		}
	}

	function canChangeProfile() {
		return $this->profileWrite;
	}

	function canChangePassword() {
		return $this->passwdWrite;
	}

	function canChangeForward() {
		return $this->forwardWrite;
	}

	function canChangeVacation() {
		return $this->vacationWrite;
	}

	function canChangeJunkMail() {
		return $this->junkmailWrite;
	}

	function canCreateAliases() {
		return $this->aliasCreate;
	}

	function canAddRemoteMember() {
		return $this->aliasAddRemote;
	}

	function canAddAliasMember() {
		return $this->aliasAddAlias;
	}

	function usingRBL() {
		return $this->doRBLLookups;
	}
}

?>
