<?php
/*
 * Copyright (C) 2001-2002 Caldera International, Inc.  All Rights Reserved.
 */

class UserListQuota extends ObjectTable {

	function UserListQuota($varArray, $id) {
		global $session_domain;

		$this->list = '';
		$cmd = new Command('msguserlistall');
		$cmd->add_arg('--domain', $session_domain);
		$cmd->add_arg('--uid --mail --fullname --quotalimit --quotaused');
		$cmd->run();
		$lines = $cmd->getOutput();

		unset($this->list);

		// Populate the list 
		if ($lines != '') foreach ($lines as $line) {

			$line = trim($line);

			if ($line == '') {
				if ($user != '') {
					$this->list[] = $user;
					$user = '';
				}
				continue;
			}

			if ($user == '' && $line != '') {
				$user = new User();
			}

			list($attr, $value) = split(':', $line, 2);
			$user->setAttr($attr, $value);
		}

		// Apply filter if any
		$varArray['listClass'] = 'UserListQuota';
		$this->setFilter($varArray, $id);
		if (isset($this->filter)) $this->applyFilter($this->filter);

		// now that we've got the filter, let's switch id's
		$id = 'quota' . $id;

		// Setup the table
		if (Table::isActive($varArray, $id)) {
			$table = Table::getActive();
			$table->setParams($varArray, $this->count());
		} else {
			$table = new Table($id);

			$name = getIntlMsg('MSG_USERS_HEADING_EMAIL');
			$table->addDataColumn($name, Column::ASCEND(), 'mail',
				"return Go('userquota.php?uid=%%ROWID%%')");

			$name = getIntlMsg('MSG_USERS_HEADING_QUOTA');
			$table->addDataColumn($name, Column::ASCEND(), 'quota');

			$name = getIntlMsg('MSG_USERS_HEADING_QUOTA_USED');
			$table->addDataColumn($name, Column::ASCEND(), 'used');

			$name = getIntlMsg('MSG_USERS_HEADING_QUOTA_FREE');
			$table->addDataColumn($name, Column::ASCEND(), 'free');

			$name = getIntlMsg('MSG_USERS_HEADING_QUOTA_PERCENT');
			$table->addDataColumn($name, Column::ASCEND(), 'percent');

			$table->setParams($HTTP_GET_VARS, $this->count());
			$table->setSortedColumn($table->firstSortedColumn());
		}

		// Apply sorting
		$attr = $table->getSortAttrName($table->getSortedColumn());
		$dir  = $table->getColumnSortDir($table->getSortedColumn());
		$this->sort(User::comparisonFunctionName($attr ,$dir));

		// Save table
		$this->table = $table;
		$this->table->save();
	}

	function getListValue($index, $col = '') {
		global $session_lang;

		// if now $col then return the row value (unique id)
		if ($col === '') return $this->list[$index]->getUID();

		$x = $this->list[$index];

		switch ($col) {
			case '0':
			case 'mail':
				$retval = $x->shortmail();
				break;
			case '1' :
			case 'quota':
				$retval = $x->getQuota();
				if ($retval === false) {
					$retval = getIntlMsg(
						'MSG_USERS_VALUE_QUOTA_NONE');
				}
				break;
			case '2' :
			case 'used':
				$retval = $x->getQuotaUsed();
				if ($retval === false) {
					$retval = getIntlMsg(
						'MSG_USERS_VALUE_QUOTA_NA');
				}
				break;
			case '3' :
			case 'free':
				$retval = $x->getQuotaFree();
				if ($retval === false) {
					$retval = getIntlMsg(
						'MSG_USERS_VALUE_QUOTA_NA');
				}
				break;
			case '4' :
			case 'percent':
				$retval = $x->getQuotaPercent();
				if ($retval === false) {
					$retval = getIntlMsg(
						'MSG_USERS_VALUE_QUOTA_NA');
				}
				break;
			default:
				$retval = '';
				break;
		}
		return (string) $retval;
	}

	/**********************************************************************
	 * class function getSearchAttrs()
	 *
	 * return an array containing the attributes that can be used
	 * to filter the results displayed in the table. The array has the
	 * internally meaningful attribute as the key and the displayable
	 * attribute name as the value;
	 **********************************************************************
	 */
	function getSearchAttrs() {
		return array(
			'mail' => getIntlMsg('MSG_USERS_HEADING_EMAIL'),
			'quota' => getIntlMsg('MSG_USERS_HEADING_QUOTA'),
			'used' => getIntlMsg('MSG_USERS_HEADING_USED'),
			'free' => getIntlMsg('MSG_USERS_HEADING_FREE'),
			'percent' => getIntlMsg('MSG_USERS_HEADING_PERCENT')
		);
	}


}

?>
